/**
	$Id: InfoRequestHandler.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent.handlers;

import java.net.Socket;

import net.sf.basedb.clients.jobagent.Agent;
import net.sf.basedb.util.jobagent.JobAgentInfo;
import net.sf.basedb.util.jobagent.RequestHandler;

/**
	This is a request handler for the <code>info</code> and <code>status</code>
	commands. It creates a {@link JobAgentInfo} object and uses it's
	<code>toString()</code> method to return the answer.
 
	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/
public class InfoRequestHandler
	implements RequestHandler
{

	private final Agent agent;
	public InfoRequestHandler(Agent agent)
	{
		this.agent = agent;
	}
	
	/*
		From the RequestHandler interface
		-------------------------------------------
	*/
	public String handleCmd(Socket incoming, String cmd)
	{
		String answer = null;
		if ("info".equals(cmd))
		{
			Runtime runtime = Runtime.getRuntime();
			// TODO - Get this info when it can be done by the JVM or use external calls
			// Useful link: http://builder.com.com/5100-6370_14-6067049.html
			Integer cpu = null; // (int)(Math.random()*100);
			Long totalMemory = null; // runtime.maxMemory();
			Long usedMemory = null; //runtime.totalMemory() - runtime.freeMemory();
			JobAgentInfo info = new JobAgentInfo(!agent.isRunning(), cpu, usedMemory, totalMemory, agent.getRunningJobs());
			answer = "OK\n"+info.toString();
		}
		else if ("status".equals(cmd))
		{
			JobAgentInfo info = new JobAgentInfo(!agent.isRunning(), null, null, null, null);
			answer = "OK\n"+info.toString();
		}
		else
		{
			answer = "FAILED Unhandled command: " + cmd;
		}
		return answer;
	}
	// -------------------------------------------

}
