/*
	$Id: Columns.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
	This tag is a place-holder for {@link Header &lt;tbl:header&gt;} tags.
	It has no attributes and must be placed inside a {@link Data &lt;tbl:data&gt;}
	tag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:columns&gt;
      &lt;tbl:header ... &gt;
   &lt;/tbl:columns&gt;
</pre>
	<p>
	The <code>&lt;tbl:columns&gt;</code> may contain the following subtags:
	<ul>
	<li>{@link Header &lt;tbl:header&gt;}: Adds a column header to the table
	</ul>

	@author Nicklas
	@version 2.0
*/
public class Columns
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = 8857257637171219032L;
	private Table table;
	private Data data;
	
	/*
		Package methods
	*/
	Table getTable()
	{
		return table;
	}
	Data getData()
	{
		return data;
	}

	/*
		Display methods
	*/
	public int doStartTag()
		throws JspException
	{
		if (!(getParent() instanceof Data)) throw new JspTagException("Tag <tbl:columns> must be inside a <tbl:data> tag");
		data = (Data)getParent();
		table = data.getTable();
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag()
		throws JspException
	{
		try
		{
			table.writeColumns();
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}

