/*
	$Id: NameablePluginAdaptor.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.util;

import java.util.Set;

import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PluginDefinition;



/**
	This class wraps a <code>Nameable</code> object around a 
	<code>PluginDefinition</code>. Calls to <code>setName</code>
	and <code>setDescription</code> are ignored.
	
	@author Nicklas
	@version 2.0
*/
public class NameablePluginAdaptor
	implements Nameable
{

	private final PluginDefinition plugin;
	
	public NameablePluginAdaptor(PluginDefinition plugin)
	{
		if (plugin == null) throw new InvalidUseOfNullException("PluginDefinition cannot be null");
		this.plugin = plugin;
	}

	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return plugin.getName();
	}
	public String getDescription()
	{
		return plugin.getDescription();
	}
	/**
		Ignored.
	*/
	public void setDescription(String description) 
		throws PermissionDeniedException, InvalidDataException
	{}
	/**
		Ignored.
	*/
	public void setName(String name) 
		throws PermissionDeniedException, InvalidDataException
	{}
	// -------------------------------------------

	/*
		From the Identifable interface
		-------------------------------------------
	*/
	public int getId()
	{
		return plugin.getId();
	}
	public Item getType()
	{
		return plugin.getType();
	}
	public int getVersion()
	{
		return plugin.getVersion();
	}
	// -------------------------------------------

	/*
		From the AccessControlled interface
		-------------------------------------------
	*/
	public void checkPermission(Permission permission) 
		throws PermissionDeniedException
	{
		plugin.checkPermission(permission);
	}

	public boolean hasPermission(Permission permission)
	{
		return plugin.hasPermission(permission);
	}
	public Set<Permission> getPermissions()
	{
		return plugin.getPermissions();
	}
	// -------------------------------------------

}

