/*
	$Id: Annotatable.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

/**
	This interface is implemented by all items which can be annotated.
	Annotations are extra values attached to an item. Usually these values
	are used in the analysis of an experiment.
	
	@author Nicklas
	@version 2.0
	@see AnnotationSet
	@see Annotation
	@see AnnotationType
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public interface Annotatable
	extends Identifiable
{

	/**
		Get the annotation set containing the annotations for this item.
		If the item doesn't have any annotations a new annotation set is created 
		and automatically saved to the database when {@link DbControl#commit()} is 
		called. To check if an item has annotations without creating a new 
		annotation set use the {@link #isAnnotated()} method.
		@return An <code>AnnotationSet</code>
		@throws PermissionDeniedException If the logged in user doesn't have
			enough permissions
		@throws BaseException If there is another error
	*/
	public AnnotationSet getAnnotationSet()
		throws PermissionDeniedException, BaseException;

	/**
		Check if this item has an annotation set. The annotation set may be
		empty.
		@return TRUE if an annotation set exists, FALSE otherwise
	*/
	public boolean isAnnotated();
	
	/**
		Remove all annotations from this item, by deleting the annotation set.
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission
		@throws BaseException If there is another error
	*/
	public void removeAnnotations()
		throws PermissionDeniedException, BaseException;

	/**
		Get all parents objects which are annotatable and the logged in
		user has read permission to. If the item doesn't have any annotatable
		parents, it may return null or an empty set. The method should only return
		the immediate parent(s), not parents to parents, etc.

		@return A set containing annotatable items, or null
		@throws BaseException If there is an error
	*/
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException;
	
	/**
		A protcol used in the creation of an item that is used
		to attach annotations for the protocol parameters.
		@return A protocol or null if the protocol is unknown
		@since 2.2
	*/
	public Protocol getProtocol();
}
