/*
	$Id: DateParameterType.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.DateParameterValueData;

import java.util.Date;
import java.util.List;

/**
	This class represent a parameter type that is a date.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class DateParameterType
	extends ParameterType<Date>
{
	/**
		Create a new date parameter type, allowing null values.
	*/
	public DateParameterType()
	{
		this(null, false);
	}
	
	/**
		Create a new date parameter type.
	 	@param defaultValue Value to use as default. Null is allowed. 
		@param notNull FALSE if nulls values are allowed, TRUE otherwise
	*/
	public DateParameterType(Date defaultValue, boolean notNull)
	{
		super(Date.class, defaultValue, notNull);
	}
	
	/**
	 	Creates a new DateParameterType
	  	@param defaultValue The default value for this parameter type.
	  	@param notNull FALSE if this parameter type is allowed to be null, otherwise TRUE.
	  	@param multiplicity How many values that can be tied to this parameter.
	  	@param width Width of the input field for this parameter type in the GUI.
	  	@param height Height of the input field for this parameter type in the GUI.
	  	@param items A list of allowed items for this parameter type.
	*/
	public DateParameterType(Date defaultValue, boolean notNull, 
		int multiplicity, int width, int height, List<Date> items)
	{
		super(Date.class, defaultValue, notNull, multiplicity, width, height, items);
	}

	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "DateParameterType";
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Always validate.
	*/
	@Override
	void validateValue(String name, Date value)
		throws InvalidDataException
	{}
	
	/**
		Create a new {@link DateParameterValueData} object.
	*/
	@Override
	DateParameterValueData newParameterValueData()
	{
		return new DateParameterValueData();
	}
	// -------------------------------------------


}
