/*
	$Id: IntensityFormula.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
	Holds information about the formulas used to calculate intensities from
	a raw data spot. An instance of this object holds a set of formulas
	corresponding to the number of channels for a raw data type. 

	@see RawDataType
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class IntensityFormula
{

	private final String name;
	private final String title;
	private final String description;
	private final String[] expressions;

	/**
		Create a new intensity formula.
		See the getter methods for a description of the parameters.
	*/
	IntensityFormula(String name, String title, String description, String[] expressions)
	{
		this.name = name;
		this.title = title;
		this.description = description;
		this.expressions = expressions;
	}

	/**
		Get the name of the intensity formula.
		@return A string with the name
	*/
	public String getName()
	{
		return name;
	}
	/**
		Get the title of the intensity formula. The title is not used
		by the core but can be used by client applications to display selection lists.
		@return A string with the title
	*/
	public String getTitle()
	{
		return title;
	}
	/**
		Get a description of the intensity formula. The description
		is not used by the core but can be used by client applications.
		@return A string with the description
	*/
	public String getDescription()
	{
		return description;
	}
	
	/**
		Get the expressions used to calculate the intensities. The expression
		for channel 1 is located in array position 0, for channel 2 in array
		postion 1, and so on.
	*/
	public String[] getExpressions()
	{
		return expressions;
	}

}

