/*
	$Id: ItemModifiedException.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
	This exception is thrown when trying to save an item to the database
	that has been updated (or deleted) by another process in the meantime.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class ItemModifiedException
	extends PermissionDeniedException
{

	/**
		
	*/
	private static final long serialVersionUID = 6443430913361934576L;
	/**
		Creates a new <code>ItemModifiedException</code>. The error
		message produced will look like:
		<code>The item User[id=325] was modified by another transaction.</code>

		@param what A description of what was modified, for 
			example User[Id=325]
	*/
	public ItemModifiedException(String what)
	{
		super("The item "+what+" was modified by another transaction.");
	}
	
	/**
		Create a new <code>ItemModifiedException</code> trying to find the item
		information from the Hibernate exception. Normally the
		item is specified in the message like:
		<code>[net.sf.basedb.core.data.XxxxData#id]</code>.
	 	@param ex Exception generated by Hibernate. 
	*/
	public ItemModifiedException(org.hibernate.StaleStateException ex)
	{
		super("The item "+findWhat(ex)+" was modified by another transaction.", ex);
	}
	
	/**
		A pattern to extract the <code>Xxxx</code> and <code>id</code> parts
		from a string containing <code>[net.sf.basedb.core.data.XxxxData#id]</code>.
	*/
	private static final Pattern FIND_WHAT = Pattern.compile("\\[net\\.sf\\.basedb\\.core\\.data\\.(\\w+)Data\\#(\\d+)\\]");
	private static String findWhat(org.hibernate.StaleStateException ex)
	{
		String msg = ex.getMessage();
		String what = "Unknown item[id=unknown]";
		Matcher m = FIND_WHAT.matcher(msg);
		if (m.find())
		{
			what = m.group(1)+"[id="+m.group(2)+"]";
		}
		return what;
	}
}

