/*
	$Id: Operator.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Expression;

import java.util.Map;
import java.util.HashMap;

/**
	@author Nicklas, Martin
	@version 2.0
*/
public enum Operator
{
	AND(1, "AND", "AND", false, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	OR(2, "OR", "OR", false, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	NOT(3, "NOT", "NOT", false, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	EQ(4, "=", "=", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.eq(lvalue, rvalue);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	NEQ(5, "!=", "!=", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.neq(lvalue, rvalue);
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	LT(6, "<", "<", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.lt(lvalue, rvalue);
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	LTEQ(7, "<=", "<=", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.lteq(lvalue, rvalue);
		}
		
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	GT(8, ">", ">", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.gt(lvalue, rvalue);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	GTEQ(9, ">=", ">=", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.gteq(lvalue, rvalue);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},

	LIKE(10, "LIKE", "LIKE", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.like(lvalue, rvalue);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},
	
	NOTLIKE(11, "NOT LIKE", "NOT LIKE", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.not(Restrictions.like(lvalue, rvalue));
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
	},
	
	/**
	 	@since 2.6
	 */
	IN(12, "IN", "IN", true, true)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			Expression[] expressionArray = {rvalue};
			return getRestriction(lvalue, expressionArray);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			return Restrictions.in(lvalue, rvalues);
		}
	},
	
	/**
	 	@since 2.6
	 */
	NOTIN(13, "NOT IN", "NOT IN", true, true)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			Expression[] expressionArray = {rvalue};
			return getRestriction(lvalue, expressionArray);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			return Restrictions.not(Restrictions.in(lvalue, rvalues));
		}
	},
	
	/**
	 	@since 2.6
	 */
	LIKE_IN(14, "LIKE", "LIKE", true, true)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			Expression[] expressionArray = {rvalue};
			return getRestriction(lvalue, expressionArray);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			return Restrictions.like_in(lvalue, rvalues);
		}
	},

	/**
	 	@since 2.6
	 */
	NOTLIKE_IN(15, "NOT LIKE", "NOT LIKE", true, true)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			Expression[] expressionArray = {rvalue};
			return getRestriction(lvalue, expressionArray);
		}

		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			return Restrictions.notLike_in(lvalue, rvalues);
		}
	},
	
	/**
		@since 2.8
	*/
	RLIKE(16, "~", "~", true, false)
	{
		public Restriction getRestriction(Expression lvalue, Expression rvalue)
			throws InvalidDataException
		{
			return Restrictions.rlike(lvalue, rvalue);
		}
		public Restriction getRestriction(Expression lvalue, Expression rvalues[])
			throws InvalidDataException
		{
			throw new InvalidDataException("Not allowed");
		}
		public String getSqlSymbol()
		{
			return HibernateUtil.getDbEngine().rlike("", "");
		}
	};

	private static final Map<Integer, Operator> valueMapping = new HashMap<Integer, Operator>();
	
	static
	{
		for(Operator op : Operator.values())
		{
			Operator o = valueMapping.put(op.getValue(), op);
			assert o == null : "Another operator with the value "+op.getValue()+" already exists";
		}
	}

	private final int value;
	private final String symbol;
	private final String sqlSymbol;
	private final boolean expressionOperator;
	private final boolean isListOperator;

	private Operator(int value, String symbol, String sqlSymbol, boolean expressionOperator, boolean isListOperator)
	{
		this.value = value;
		this.symbol = symbol;
		this.sqlSymbol = sqlSymbol;
		this.expressionOperator = expressionOperator;
		this.isListOperator = isListOperator;
	}

	public String toString()
	{
		return symbol;
	}
	
	/**
		Get the integer value that is used when storing an operator in the database.
	 	@return an Integer 
	*/
	public int getValue()
	{
		return value;
	}
	
	/**
		Get the symbol for this operator, useful for display in client applications.
	 	@return a String object 
	*/
	public String getSymbol()
	{
		return symbol;
	}

	/**
		Get the SQL symbol for this operator. This can be used when creating 
		SQL/HQL queries.
		@return A String object
	*/
	public String getSqlSymbol()
	{
		return sqlSymbol;
	}
	
	public boolean isExpressionOperator()
	{
		return expressionOperator;
	}
	
	public abstract Restriction getRestriction(Expression lvalue, Expression rvalue)
		throws InvalidDataException;

	public abstract Restriction getRestriction(Expression lvalue, Expression rvalues[])
		throws InvalidDataException;

	
	/**
		Get the <code>Operator</code> object when you know the integer code.
	 	@param value The integer code 
	 	@return an Operator object. 
	*/
	public static Operator fromValue(int value)
	{
		Operator op = valueMapping.get(value);
		assert op != null : "operator == null for value "+value;
		return op;
	}

	/**
	 	Gets if the operator should be used with a list or not.
	 	The list items should be separated with '|'.
	 	@return TRUE if it should, FALSE otherwise.
	 	@since 2,6
	 */
	public boolean isListOperator()
	{
		return isListOperator;
	}
}
