/*
	$Id: Version.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;

/**
	Get information about the BASE version.
	<p>
	The version information is stored in the <code>base.version</code> file. 

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class Version
{
	private static String major;
	private static String minor;
	private static String maintenance;
	private static int build;
	
	static
	{
		init();
	}
	
	/**
		Loads the settings from the configuration file.
		@throws ConfigurationException This exception is thrown if the
			configuration settings cannot be loaded
	*/
	static synchronized void init()
		throws ConfigurationException
	{
		try
		{
			URL baseConfig = Version.class.getResource("/base.version");
			InputStream is = baseConfig == null ? null : baseConfig.openStream();
			if (is == null)
			{
				Application.getLogger().warn("File 'base.version' not found.");
			}
			Properties config = new Properties();
			config.load(is);
			is.close();
			
			major = config.getProperty("major");
			minor = config.getProperty("minor");
			maintenance = config.getProperty("maintenance");
			build = parseBuildNumber(config.getProperty("build"));
		}
		catch (IOException ex)
		{
			Application.getLogger().warn("Error while loading version information from '/base.version'.", ex);
		}
	}
	
	/**
		Get the major version.
	*/
	public static String getMajor()
	{
		return major;
	}
	
	/**
		Get the minor version.
	*/
	public static String getMinor()
	{
		return minor;
	}
	/**
		Get the maintentance version.
	*/
	public static String getMaintenance()
	{
		return maintenance;
	}
	
	/**
		Get the build version.
	*/
	public static int getBuild()
	{
		return build;
	}
	
	public static String getVersion()
	{
		return major+"."+minor+"."+maintenance + "(build " + build + ")";
	}

	/**
		Find the first digits in a string and return those as the build number.
	*/
	private static int parseBuildNumber(String buildNumber)
	{
		int build = 0;
		Pattern p = Pattern.compile("(\\d+)");
		Matcher m = p.matcher(buildNumber);
		if (m.find())
		{
			build = Integer.parseInt(m.group());
		}
		return build;
	}

}
