/*
	$Id: ArrayDesignBlockData.java 4517 2008-09-11 20:11:02Z jari $
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This holds information about a block in an arraydesign. It must be lazy because 
	features are loaded in a stateless session.
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.ArrayDesignBlock
	@see <a href="../../../../../../html/developerdoc/api/data_api.arrays.html">Array LIMS - arrays overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`ArrayDesignBlocks`" lazy="false"
*/
public class ArrayDesignBlockData 
	extends BasicData
{

	public ArrayDesignBlockData()
	{}
	
	private int metaGridX;

	/**
		Get the row coordinate of this block.
		@hibernate.property column="`meta_grid_x`" type="int" not-null="true" update="false"
	*/
	public int getMetaGridX()
	{
		return metaGridX;
	}

	public void setMetaGridX(int metaRow)
	{
		this.metaGridX = metaRow;
	}

	private int metaGridY;

	/**
	 Get the column coordinate of this block.
	 @hibernate.property column="`meta_grid_y`" type="int" not-null="true" update="false"
	 */
	public int getMetaGridY()
	{
		return metaGridY;
	}

	public void setMetaGridY(int metaColumn)
	{
		this.metaGridY = metaColumn;
	}

	private int blockSizeX;

	/**
	 Get the maximum number of rows in this block.
	 @hibernate.property column="`block_size_x`" type="int" not-null="true" 
	 */
	public int getBlockSizeX()
	{
		return blockSizeX;
	}

	public void setBlockSizeX(int rows)
	{
		this.blockSizeX = rows;
	}

	private int blockSizeY;

	/**
	 Get the maximum number of columns in this block.
	 @hibernate.property column="`block_size_y`" type="int" not-null="true" 
	 */
	public int getBlockSizeY()
	{
		return blockSizeY;
	}

	public void setBlockSizeY(int columns)
	{
		this.blockSizeY = columns;
	}
	
	private int originX;
	/**
	 Get the distance from the left edge of the slide to the top left spot in the block.
	 @hibernate.property column="`origin_x`" type="int" not-null="true" 
	 */
	public int getOriginX()
	{
		return originX;
	}

	public void setOriginX(int originX)
	{
		this.originX = originX;
	}
	
	private int originY;
	
	/**
	 Get the distance from the top edge of the slide to the top left spot in the block.
	 @hibernate.property column="`origin_y`" type="int" not-null="true" 
	 */
	public int getOriginY()
	{
		return originY;
	}

	public void setOriginY(int originY)
	{
		this.originY = originY;
	}
	
	private int spacingX;
	
	/**
	 Get the x spacing.
	 @hibernate.property column="`spacing_x`" type="int" not-null="true" 
	 */
	public int getSpacingX()
	{
		return spacingX;
	}

	public void setSpacingX(int spacingX)
	{
		this.spacingX = spacingX;
	}
	
	private int spacingY;
	
	/**
	 Get the y spacing.
	 @hibernate.property column="`spacing_y`" type="int" not-null="true" 
	 */	
	public int getSpacingY()
	{
		return spacingY;
	}

	public void setSpacingY(int spacingY)
	{
		this.spacingY = spacingY;
	}

	private int featureDiameter;
	/**
		The expected diameter of spots.
	 	@hibernate.property column="`feature_diameter`" type="int" not-null="true" 
	*/	
	public int getFeatureDiameter()
	{
		return featureDiameter;
	}
	public void setFeatureDiameter(int featureDiameter)
	{
		this.featureDiameter = featureDiameter;
	}
	
	private int blockNumber;

	/**
	 Get the block number of this block.
	 @hibernate.property column="`block_number`" type="int" not-null="true" update="false"
	 */
	public int getBlockNumber()
	{
		return blockNumber;
	}

	public void setBlockNumber(int blockNumber)
	{
		this.blockNumber = blockNumber;
	}

	private Set<FeatureData> features;
	/**
		Used by Hibernate to link with features. This is the inverse end.
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`arraydesignblock_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.FeatureData"
	*/
	Set<FeatureData> getFeatures()
	{
		return features;
	}
	void setFeatures(Set<FeatureData> features)
	{
		this.features = features;
	}

	private ArrayDesignData arrayDesign;

	/**
	 Used by Hibernate to link with ArrayDesign.
	 @hibernate.many-to-one column="`arraydesign_id`" not-null="true" outer-join="false" update="false"
	 */
	public ArrayDesignData getArrayDesign()
	{
		return arrayDesign;
	}

	public void setArrayDesign(ArrayDesignData arrayDesign)
	{
		this.arrayDesign = arrayDesign;
	}
}
