/*
  $Id: BioMaterialListData.java 4679 2008-12-02 08:37:18Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.HashSet;
import java.util.Set;

/**
	This represents an arbitrary collection of biomaterials.
	
	@author Nicklas
	@version 2.10
	@see net.sf.basedb.core.BioMaterialList
	@see <a href="../../../../../../html/developerdoc/api/data_api.biomaterials.html">Biomaterials overview</a>
	@hibernate.class table="`BioMaterialLists`" lazy="false"
	@base.modified $Date: 2008-12-02 09:37:18 +0100 (Tue, 02 Dec 2008) $
*/
public class BioMaterialListData
	extends CommonData
{
	public BioMaterialListData()
	{}

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for this <code>ReporterList</code> item.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	private int memberType;
	/**
		Get the type of (biomaterial) items this list can hold. Not possible to change
		after creation.
		@hibernate.property column="`member_type`" type="int" not-null="true" update="false"
	*/
	public int getMemberType()
	{
		return memberType;
	}
	public void setMemberType(int memberType)
	{
		this.memberType = memberType;
	}
	
	private int size;
	/**
		The number of biomaterials in the list.
		@hibernate.property column="`size`" type="int" not-null="true"
	*/
	public int getSize()
	{
		return size;
	}
	public void setSize(int size)
	{
		this.size = size;
	}

	private Set<BioMaterialData> bioMaterials;
	/**
		Get the set that manages which biomaterials are part of this list.
		@hibernate.set table="`BioMaterialListMembers`" lazy="true"
		@hibernate.collection-key column="`list_id`"
		@hibernate.collection-many-to-many column="`biomaterial_id`" class="net.sf.basedb.core.data.BioMaterialData"
	*/
	public Set<BioMaterialData> getBioMaterials()
	{
		if (bioMaterials == null) bioMaterials = new HashSet<BioMaterialData>();
		return bioMaterials;
	}
	void setBioMaterials(Set<BioMaterialData> bioMaterials)
	{
		this.bioMaterials = bioMaterials;
	}
	
}
