/*
  $Id: PlatformVariantData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class hols information about variants of experimental platforms.
	
	@author Nicklas
	@version 2.5
	@see net.sf.basedb.core.PlatformVariant
	@see <a href="../../../../../../html/developerdoc/api/data_api.platforms.html">Data API - Experimental platforms</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`PlatformVariants`" lazy="false"
*/
public class PlatformVariantData
	extends BasicData
	implements NameableData, RemovableData
{

	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	/*
		-------------------------------------------		
	*/

	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for the item. 
		@return The external id of the item
		@hibernate.property column="`external_id`" type="string" length="255" 
			not-null="true" update="false" unique="true"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}
	
	private boolean fileOnly;
	/**
		If this platform variant can only be used with files.
		@hibernate.property column="`file_only`" type="boolean" not-null="true" update="false"
	*/
	public boolean isFileOnly()
	{
		return fileOnly;
	}
	public void setFileOnly(boolean fileOnly)
	{
		this.fileOnly = fileOnly;
	}

	private String rawDataType;
	/**
		The raw data type this platform variant is locked to, or null.
		Only used if fileOnly=false
		@hibernate.property column="`rawdatatype`" type="string" length="255" 
			not-null="false" update="false"
	*/
	public String getRawDataType()
	{
		return rawDataType;
	}
	public void setRawDataType(String rawDataType)
	{
		this.rawDataType = rawDataType;
	}

	private int channels;
	/**
		The number of channels this platform variant uses. Only used
		if fileOnly=true
		@hibernate.property column="`channels`" type="int" not-null="true" update="false"
	*/
	public int getChannels()
	{
		return channels;
	}
	public void setChannels(int channels)
	{
		this.channels = channels;
	}
	
	private PlatformData platform;
	/**
		The parent platform of this variant.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`platform_id`" not-null="true"
	*/
	public PlatformData getPlatform()
	{
		return platform;
	}
	public void setPlatform(PlatformData platform)
	{
		this.platform = platform;
	}
	
	private Set<PlatformFileTypeData> fileTypes;
	/**
		The additional file types that can be used on this platform variant.
		This is the inverse end. 
		@see PlatformFileTypeData#getVariant()
		@hibernate.set lazy="true" cascade="delete" inverse="true"
		@hibernate.collection-key column="`variant_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.PlatformFileTypeData"
	*/
	Set<PlatformFileTypeData> getFileTypes()
	{
		return fileTypes;
	}
	void setFileTypes(Set<PlatformFileTypeData> fileTypes)
	{
		this.fileTypes = fileTypes;
	}

}
