/*
  $Id $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds information the symbol that are used with units.
	A symbol must be unique within a given {@link QuantityData}. To make
	it easier for user to input we will also register alternative symbols 
	for units. For example:
	
	<pre class="code">
Temperature: C -- C
Surface area: m -- m2
</pre>
	
	@author Nicklas
	@version 2.9
	@base.modified $Date $
	@hibernate.class table="`UnitSymbols`" lazy="false"
*/
public class UnitSymbolData
	extends BasicData
{

	private QuantityData quantity;
	/**
		Get the quantity this symbol belongs to.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`quantity_id`" not-null="true" unique-key="uniquesymbol"
	*/
	public QuantityData getQuantity()
	{
		return quantity;
	}
	public void setQuantity(QuantityData quantity)
	{
		this.quantity = quantity;
	}
	
	private String symbol;
	/**
		The symbol, which must be unique for a given quantity.
		@hibernate.property type="string"
		@hibernate.column name="`symbol`" length="255" not-null="true" unique-key="uniquesymbol"
	*/
	public String getSymbol()
	{
		return symbol;
	}
	public void setSymbol(String symbol)
	{
		this.symbol = symbol;
	}

	private UnitData unit;
	/**
		Get the unit this symbol belongs to.
		@hibernate.many-to-one column="`unit_id`" update="false" not-null="true" outer-join="false"
	*/
	public UnitData getUnit()
	{
		return unit;
	}
	public void setUnit(UnitData unit)
	{
		this.unit = unit;
	}
	
}
