/*
	$Id: HqlElementsExpression.java 4620 2008-10-31 08:54:10Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;

/**
	An expression representing the elements collection of a
	collection of a Hibernate entity: elements(alias.property).
	
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#HQL}.

	@author Nicklas
	@version 2.0
	@see Hql#elements(String, String)
	@base.modified $Date: 2008-10-31 09:54:10 +0100 (Fri, 31 Oct 2008) $
*/
public class HqlElementsExpression
	implements Expression
{
	private final String alias;
	private final String property;

	HqlElementsExpression(String alias, String property)
	{
		assert property != null : "property == null";
		this.alias = alias;
		this.property = property;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.HQL)
		{
			return "elements("+(alias == null ? query.getRootAlias() : alias) + 
				(property == null ? "" : "." +property.replace("#", "."))+")";
		}
		else
		{
			throw new UnsupportedOperationException("'elements' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "elements(" + (alias == null ? "" : alias +".") + property + ")";
	}
	// -------------------------------------------
}
