/*
	$Id: HqlEntityExpression.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.data.BasicData;

/**
	An expression representing a Hibernate entity.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#HQL}.

	@author Nicklas
	@version 2.0
	@see Hql#entity(BasicItem)
	@see Hql#entity(BasicData)
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class HqlEntityExpression
	implements Expression
{
	private final int id;
	private final String item;

	HqlEntityExpression(BasicItem item)
	{
		this.id = item.getId();
		this.item = item.toString();
	}
	
	/**
		@since 2.2.2
	*/
	HqlEntityExpression(BasicData data)
	{
		this.id = data.getId();
		this.item = data.toString();
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.HQL)
		{
			return Integer.toString(id);
		}
		else
		{
			throw new UnsupportedOperationException("'entity' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return item;
	}
	// -------------------------------------------
}
