/**
	$Id: HqlExpression.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core.query;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;

import java.util.Collection;

/**
 	A class holding an expression or a restriction where a prefix	
 	can be used to represent a root alias in a HQL. 
 	I.e. $name = 'somename' could be used for all
	items that have a name property. '$' will be replaced with the 
	right root alias when the restriction is used in a query, 
	$name = 'somename' will be usr.name = 'somename' in a query 
	for the user table. 
 	
  	@author Martin
  	@version 2.5
 */
public class HqlExpression 
	implements Expression, Restriction
{
	/**
	 	The restriction or expression as a string.
	 */
	private final String expression;
	
	/**
	 	Prefix used to indentify where to put 
	 	the root alias.
	 */
	private final String prefix;
	
	
	HqlExpression(String expression, String prefix)
	{
		this.expression = expression;
		this.prefix = prefix;
	}
	
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}

	/*
	  	@see net.sf.basedb.core.query.QueryElement#toQl(net.sf.basedb.core.query.Query, net.sf.basedb.core.DbControl)
	 */
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		if (query.getQueryType() == QueryType.HQL)
		{	
			return (prefix != null ? expression.replace(prefix, query.getRootAlias() + ".") : expression);
		}
		else
		{
			throw new UnsupportedOperationException("'hql expression' operation not supported by " + query.getQueryType() + " query.");
		}
	}
	
	/**
	 	Returns the expression field.
	 */
	public String toString()
	{
		return expression;
	}

}
