/*
	$Id: ParameterExpression.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.BaseException;

/**
	Parameter expression. Used to be able to set a value in the
	query from the query object.

	@author Nicklas
	@version 2.0
	@see Expressions#parameter(String, Object, Type)
	@see Query#setParameter(String, Object, Type)
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class ParameterExpression
	implements Expression
{
	private final String name;
	private final Object defaultValue;
	private final Type valueType;

	ParameterExpression(String name, Object defaultValue, Type valueType)
	{
		assert name != null : "name == null";
		this.name = name;
		this.defaultValue = defaultValue;
		this.valueType = valueType;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (defaultValue != null && !query.hasParameterValue(name))
		{
			query.setParameter(name, defaultValue, valueType);
		}
		return ":" + name;
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return ":" + name + (defaultValue == null ? "" : "(" + defaultValue + ")");
	}
	// -------------------------------------------
}
