/*
	$Id: VirtualColumnExpression.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.VirtualTable;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.BaseException;

/**

	@author Nicklas
	@version 2.0
	@see Dynamic#column(VirtualTable, VirtualColumn)
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class VirtualColumnExpression
	implements Expression
{
	private final VirtualTable table;
	private final VirtualColumn column;

	VirtualColumnExpression(VirtualTable table, VirtualColumn column)
	{
		this.table = table;
		this.column = column;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			return (table == null ? query.getRootAlias() : HibernateUtil.quote(table.getAlias())) + "." + HibernateUtil.quote(column.getColumn());
		}
		else
		{
			throw new UnsupportedOperationException("'column' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return (table == null ? "" : table.getAlias() + ".") + column.getColumn();
	}
	// -------------------------------------------
}
