/*
	$Id$

	Copyright (C) 2005 Johan Enell
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.InvalidUseOfNullException;

/**
	Util class for coordinate transformations in base.
	
	@base.modified $Date$
	@author Enell
	@version 2.0
*/
public class Coordinate
{
	
	/**
		Convert an alphanumeric coordinate to a numeric coordinate, case is unimportent.

		<table border=1 cellspacing=0 cellpadding=2>
		<tr><th>Alphanumeric coordinate</th><th>Numeric coordinate</th></tr>
		<tr><td>a</td><td>1</td></tr>
		<tr><td>...</td><td>...</td></tr>
		<tr><td>z</td><td>26</td></tr>
		<tr><td>aa</td><td>27</td></tr>
		<tr><td>...</td><td>...</td></tr>
		<tr><td>az</td><td>52</td></tr>
		<tr><td>ba</td><td>53</td></tr>
		<tr><td>...</td><td>...</td></tr>
		<tr><td>zz</td><td>702</td></tr>
		<tr><td>aaa</td><td>703</td></tr>
		</table>

		@param s The string representation of the alphanumeric coordinate.
		@return The int representation of the numeric coordinate.
	*/
	public static int alphaToNumeric(String s)
	{
		int result = 0;
		if (s == null)
		{
			throw new InvalidUseOfNullException("coordinate");
		}
		else if (s.matches("[a-zA-Z]+"))
		{
			for (int i = 0; i < s.length(); i++)
			{
				int value = Character.getNumericValue(s.charAt(s.length()-i-1))-9;
				result += value * Math.pow(26, i);
			}
		}
		else
		{
			result = Float.valueOf(s).intValue();
		}
		return result;
	}
	
	/**
		Convert a numeric coordinate to an alphanumeric coordinate. See 
		{@link #alphaToNumeric(String)} for details.
		 
		@param i The int representation of the numeric coordinate.
		@return The string representation of the alphanumeric coordinate.
		
	*/
	public static String numericToAlpha(int i)
	{
		String result = "";
		while(i > 0)
		{
			i--;
			result = (char)((i%26)+97) + result;
			i = i / 26;
		}
		return result;
	}
	
}
