/*
	$Id: NameableComparator.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Comparator;

import net.sf.basedb.core.Nameable;

/**
	An implementation of the <code>Comparator</code> interface which uses
	the name of {@link Nameable} items to compare the objects. If the names
	are equal it uses the ID:s to really check if they are equal.
	<p>
	Note: this comparator is consistent with equals.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $

*/
public class NameableComparator<T extends Nameable>
	implements Comparator<T>
{

	private final boolean descending;
	
	/**
		Create a new comparator.
		@param descending TRUE to sort objects in descending order (z--a), 
			FALSE to sort the objects in ascending order (a--z)
	*/
	public NameableComparator(boolean descending)
	{
		this.descending = descending;
	}

	public int compare(T o1, T o2)
	{
		String name1 = o1.getName();
		String name2 = o2.getName();
		int compared = name1.compareTo(name2);
		if (compared == 0) compared = o2.getId() - o1.getId();
		if (descending) compared = -compared;
		return compared;
	}

}
