/**
	$Id: HardwareLoader.java 4748 2009-02-10 14:00:00Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.ArrayBatch;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Scan;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for hardware. Hardware is loaded as
	end-point property nodes with no children. It can't be a root 
	node or a forward/reverse-loading node.
	<p>
	Validations:
	<ul>
	<li>Missing hardware: {@link Validator#MISSING_HARDWARE}
	<li>Denied hardware: {@link Validator#DENIED_HARDWARE}
	<li>Incorrect hardware type: {@link Validator#INCORRECT_HARDWARETYPE} 
	<li>Non-default hardware: {@link Validator#NONDEFAULT_HARDWARE} 
	</ul>
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-10 15:00:00 +0100 (Tue, 10 Feb 2009) $
*/
public class HardwareLoader
	extends BasicItemNodeLoader<Hardware>
{
	
	public HardwareLoader()
	{
		super(Item.HARDWARE, DENY_ROOT_NODE, 
				new NameableNameGenerator<Hardware>("hardware", "Hardware"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Hardware hardware = null;
		boolean denied = false;
		String nodeTitlePrefix = "Hardware";
		try
		{
			BasicItem parentItem = (BasicItem)parentNode.getItem(dc);
			if (parentItem instanceof Scan)
			{
				nodeTitlePrefix = "Scanner";
				hardware = ((Scan)parentItem).getScanner();
			}
			else if (parentItem instanceof Hybridization)
			{
				nodeTitlePrefix = "Hyb. station";
				hardware = ((Hybridization)parentItem).getCreationEvent().getHardware();
			}
			else if (parentItem instanceof ArrayBatch)
			{
				nodeTitlePrefix = "Print robot";
				hardware = ((ArrayBatch)parentItem).getPrintRobot();
			}
			// If there are other items with a software
			// we should add more cases here
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		
		NodeFactory<Hardware> nf = getNodeFactory(dc, context, 
			getNodeValidator(context), new NameableNameGenerator<Hardware>("hardware", nodeTitlePrefix));
		Node hardwareNode = createItemNode(nf, hardware, hardware, denied, 
				parentNode, ChildNodeDirection.NONE);
		return hardwareNode;
	}
	// -----------------------------------
	
}
