/**
	$Id: PlatformValidator.java 4764 2009-02-16 19:29:47Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.Node;

/**
	Validator implementation for platforms/variants. Validation rules:
	<ul>
	<li>Access denied: {@link Validator#DENIED_PLATFORM}
	<li>Non-default platform: {@link Validator#NONDEFAULT_PLATFORM}
	</ul>

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-16 20:29:47 +0100 (Mon, 16 Feb 2009) $
*/
public class PlatformValidator
	extends NameableNodeValidator<Platform>
{
	
	public PlatformValidator()
	{
		super(null, Validator.DENIED_PLATFORM);
	}
	
	/* 
		From BasicNodeValidator class
		-----------------------------
	*/
	/**
		Checks if the platform is the the same as
		the default for the project.
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{
		super.postValidate(dc, context, node, parentNode);
		Project project = context.getProject();
		BasicItem defaultPlatform = OverviewUtil.getProjectDefaultItem(dc, project, Project.Default.PLATFORM);
		BasicItem defaultVariant = OverviewUtil.getProjectDefaultItem(dc, project, Project.Default.PLATFORM_VARIANT);
		Platform platform = (Platform)node.getItem();
		Node variantNode = node.getChild("variant");
		PlatformVariant variant = variantNode == null ? null : (PlatformVariant)node.getItem();
		BasicItem parentItem = parentNode.getItem();
		String parentTypeName = parentNode.getItemType().toString().toLowerCase();
		boolean canChangePlatform = false;
		if (parentNode.getItemType() == Item.RAWBIOASSAY)
		{
			RawBioAssay rba = (RawBioAssay)parentItem;
			canChangePlatform = rba.getNumDbSpots() == 0;
		}
		
		if ((defaultPlatform != null && !defaultPlatform.equals(platform)) ||
			(defaultVariant != null && !defaultVariant.equals(variant)))
		{
			context.createFailure(Validator.NONDEFAULT_PLATFORM, parentNode, null,
				canChangePlatform ? new Fix("Change platform of " + parentTypeName, parentItem) : null,
				new Fix("Change default platform of project", project)
			);
		}
	}
	
	/**
		@return Always null
	*/
	@Override
	protected Fix getMissingItemFix(Node parentNode)
	{
		return null;
	}
	// --------------------------------
}
