/**
$Id: GzipFilePacker.java 4515 2008-09-11 20:08:14Z jari $

Copyright (C) 2007 Nicklas Nordborg

This file is part of BASE - BioArray Software Environment.
Available at http://base.thep.lu.se/

BASE is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

BASE is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.util.zip;

import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

/**
 	This class wraps the {@link TarFilePacker} in a GZIP stream, creating
 	a tar.gz file.

  	@author martin, Nicklas
  	@version 2.4
  	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $  	
*/
public class GzipFilePacker 
	extends TarFilePacker
{

	private GZIPOutputStream gzip;
	
	/**
		Create a new packer that compresses files with the gzip format.
	*/
	public GzipFilePacker()
	{}
	
	/*
		From the FilePacker interface
		-------------------------------------------
	*/
	/**
		@return "GZipped TAR-archive (.tar.gz)"
	*/
	public String getDescription()
	{
		return "GZipped TAR-archive (.tar.gz)";
	}
	/**
	  @return "tar.gz"
	*/
	public String getFileExtension()
	{
		return "tar.gz";
	}
	/**
	  @return "application/x-gzip"
	*/
	public String getMimeType()
	{
		return "application/x-gzip";
	}
	/**
		Wrap the output stream in a {@link GZIPOutputStream}.
	*/
	public void setOutputStream(OutputStream out)
		throws IOException
	{
		this.gzip = new GZIPOutputStream(out);
		super.setOutputStream(gzip);
	}
	/**
		Finish the GZIP stream.
	*/
	public void close()
		throws IOException
	{
		super.close();
		gzip = null;
	}
	// -------------------------------------------

}
