/**
	$Id: QueryParameter.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.info;

/**
	Hold information about a single parameter to a query. A parameter
	is represented by a colon-prefixed name in a query string,
	for example: <code>name = :name</code>.
	This class maps the parameter to an actual value.

	@author nicklas
	@version 2.5
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/
public class QueryParameter
{
	private String name;
	private String type;
	private String value;
	
	/**
		Create a new empty query parameter.
	*/
	public QueryParameter()
	{}
	
	/**
		Create a new query parameter.
		@param name The name of the parameter
		@param type The type of the parameter, as taken from the
			{@link net.sf.basedb.core.Type} enumeration
		@param value The value of the parameter
	*/
	public QueryParameter(String name, String type, String value)
	{
		this.name = name;
		this.type = type;
		this.value = value;
	}
	
	/**
		Useful constructor to create a parmeter where the value is an integer.
		@param name The name of the parameter
		@param value The value of the parameter
	*/
	public QueryParameter(String name, Integer value)
	{
		this.name = name;
		this.type = "INT";
		if (value != null) this.value = value.toString();
	}
	
	/**
		Useful constructor to create a parmeter where the value is a float.
		@param name The name of the parameter
		@param value The value of the parameter
	*/
	public QueryParameter(String name, Float value)
	{
		this.name = name;
		this.type = "FLOAT";
		if (value != null) this.value = value.toString();
	}

	/**
		Useful constructor to create a parmeter where the value is a string.
		@param name The name of the parameter
		@param value The value of the parameter
	*/
	public QueryParameter(String name, String value)
	{
		this.name = name;
		this.type = "STRING";
		if (value != null) this.value = value.toString();
	}

	/**
		Get the parameter name.
	*/
	public String getName()
	{
		return name;
	}
	
	/**
		Set the parameter name.
	*/
	public void setName(String name)
	{
		this.name = name;
	}


	/**
		Get the parameter type.
		@see net.sf.basedb.core.Type
	*/
	public String getType()
	{
		return type;
	}

	/**
		Set the parameter type.
		@see net.sf.basedb.core.Type
	*/
	public void setType(String type)
	{
		this.type = type;
	}

	/**
		Set the parameter value.
	*/
	public String getValue()
	{
		return value;
	}
	
	/**
		Set the parameter name.
	*/
	public void setValue(String value)
	{
		this.value = value;
	}

}
