/*
	$Id$

	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataBatcher;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.parser.FlatFileParser;

public class TestAnalyzePluginUtil
{
	/**
		Creates a randomly generated experiment.
	 * @param file TODO
	 * @param rawBioAssays number of {@link net.sf.basedb.core.RawBioAssay RawBioAssays} in experiment
	 * @param spots number of spots in each RawBioAssay 
	 * @param randomize Randomize imported intensity values by multiplying them with 
	 * a random value between 1+randomize and 1-randomize. Eg. if randomize=0.8, the factory is
	 * a random value between 0.8 and 1.2
	*/
	static int test_createExperiment(String file, int rawBioAssays, int spots, float randomize)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.EXPERIMENT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getNew(dc, RawDataTypes.getSafeRawDataType("genepix"));
			e.setName("TestAnalyze");
			e.setDescription("Experiment created by TestAnalyzePluginUtil.");
			dc.saveItem(e);
			for (int i = 0; i < rawBioAssays; i++)
			{
				int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, 0, 1, false);
				
				FlatFileParser parser = new FlatFileParser();
				parser.setHeaderRegexp(Pattern.compile("\"(.*)=(.*)\""));
				parser.setDataHeaderRegexp(Pattern.compile("\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*"));
				parser.setDataSplitterRegexp(Pattern.compile("\\t"));
				parser.setInputStream(FileUtil.getInputStream(new java.io.File(file)), "ISO-8859-1");
				parser.parseHeaders();
				
				RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
				rba.setName("TestAnalyzePluginUtil"+i);
				rba.setDescription("RawBioAssay created by TestAnalyzePluginUtil. Number of spots should be "+spots);
				RawDataBatcher rawDataBatcher = rba.getRawDataBatcher(null, null);
				ReporterBatcher reporterBatcher = ReporterBatcher.getNew(dc);
				for (int j = 0; parser.hasMoreData() && j < spots; j++)
				{
					FlatFileParser.Data parsedData = parser.nextData();
					RawData rawData = rawDataBatcher.newRawData();
					rawData.setBlock(Values.getInteger(parsedData.get(0), null));
					rawData.setRow(Values.getInteger(parsedData.get(2), null));
					rawData.setColumn(Values.getInteger(parsedData.get(1), null));
					rawData.setX(Values.getFloat(parsedData.get(5), null));
					rawData.setY(Values.getFloat(parsedData.get(6), null));
					String reporterId = parsedData.get(4);
					rawData.setReporter(reporterId == null ? null : reporterBatcher.getByExternalId(reporterId));
					
					float randomCh1 = (float)((2 * Math.random() - 1) * randomize + 1);
					float randomCh2 = (float)((2 * Math.random() - 1) * randomize + 1);
					
					Map<String, Object> extraData = rawData.getAllExtended();
					extraData.put("diameter", Values.getFloat(parsedData.get(7), null));
					extraData.put("ch1FgMedian", randomCh1 * Values.getFloat(parsedData.get(8), null));
					extraData.put("ch1FgMean", randomCh1 * Values.getFloat(parsedData.get(9), null));
					extraData.put("ch1FgSd", randomCh1 * Values.getFloat(parsedData.get(10), null));
					extraData.put("ch1BgMedian", randomCh1 * Values.getFloat(parsedData.get(11), null));
					extraData.put("ch1BgMean", randomCh1 * Values.getFloat(parsedData.get(12), null));
					extraData.put("ch1BgSd", randomCh1 * Values.getFloat(parsedData.get(13), null));
					extraData.put("ch1PercSd1", Values.getInteger(parsedData.get(14), null));
					extraData.put("ch1PercSd2", Values.getInteger(parsedData.get(15), null));
					extraData.put("ch1PercSat", Values.getInteger(parsedData.get(16), null));
					extraData.put("ch2FgMedian", randomCh2 * Values.getFloat(parsedData.get(17), null));
					extraData.put("ch2FgMean", randomCh2 * Values.getFloat(parsedData.get(18), null));
					extraData.put("ch2FgSd", randomCh2 * Values.getFloat(parsedData.get(19), null));
					extraData.put("ch2BgMedian", randomCh2 * Values.getFloat(parsedData.get(20), null));
					extraData.put("ch2BgMean", randomCh2 * Values.getFloat(parsedData.get(21), null));
					extraData.put("ch2BgSd", randomCh2 * Values.getFloat(parsedData.get(22), null));
					extraData.put("ch2PercSd1", Values.getInteger(parsedData.get(23), null));
					extraData.put("ch2PercSd2", Values.getInteger(parsedData.get(24), null));
					extraData.put("ch2PercSat", Values.getInteger(parsedData.get(25), null));
					
					rawDataBatcher.insert(rawData);
				}
				dc.commit();
				dc = TestUtil.getDbControl();
				dc.reattachItem(e);
				e.addRawBioAssay(rba);
			}
			dc.commit();
			id = e.getId();
		}
		catch (Throwable ex)
		{
			write("--Create experiment in TestAnalyzePluginUtil FAILED");
			ex.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}		
		return id;
	}
	
	static int test_createRootBioAssaySet(int experimentId, int formulaId)
	{
		if (!TestUtil.hasPermission(Permission.WRITE, Item.EXPERIMENT)) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			List<RawBioAssay> rawBioAssays = e.getRawBioAssays().list(dc);
			int[] rawBioAssayIds = new int[rawBioAssays.size()];
			for (int i = 0; i < rawBioAssayIds.length; i++)
			{
				rawBioAssayIds[i] = rawBioAssays.get(i).getId();
			}
			int jobId = TestExperiment.test_create_root_bioassayset_using_plugin(experimentId, 
				formulaId, rawBioAssayIds);
			TestJob.test_execute(jobId);
			ItemQuery<BioAssaySet> query = e.getBioAssaySets();
			query.order(Orders.desc(Hql.property("id")));
			id = query.iterate(dc).next().getId();
			write("--Create root BioAssaySet in TestAnalyzePluginUtil OK");
		}
		catch (Throwable ex)
		{
			write("--Create root BioAssaySet in TestAnalyzePluginUtil FAILED");
			ex.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
