/*
	$Id: TestClientDefaultSetting.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

public class TestClientDefaultSetting
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing client default setting");
		write_header();
		// Standard tests: create, load, list
		int clientId = TestClient.test_create("net.sf.basedb.clients.testcreate", true);
		int id = test_create(clientId, "fontsize", "9");
		test_duplicate(id);
		test_load(id);
		test_list(clientId, -1);

		// Extra tests:

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		TestClient.test_delete(clientId);
		write("++Testing client default setting "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int clientId, String name, String value)
	{
		if (clientId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client c = Client.getById(dc, clientId);
			ClientDefaultSetting cds = c.newDefaultSetting(name, value);
			dc.saveItem(cds);
			dc.commit();
			id = cds.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(cds);
			write_item(0, cds);
			write("--Create client default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Create client default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ClientDefaultSetting cds = ClientDefaultSetting.getById(dc, id);
			write_item(0, cds);
			write("--Load client default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Load client default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int clientId, int expectedResults)
	{
		if (clientId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client client = Client.getById(dc, clientId);
			ItemResultList<ClientDefaultSetting> l = ClientDefaultSetting.getQuery(client).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List client default settings OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List client default settings FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ClientDefaultSetting cds = ClientDefaultSetting.getById(dc, id);
			dc.deleteItem(cds);
			dc.commit();
			write("--Delete client default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Delete client default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName\tValuet\tClient");
			write("-- \t-- \t----\t------\t------");
		}
	}
	static void write_item(int i, ClientDefaultSetting cds)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+cds.getId()+"\t"+cds.getName()+"\t"+
			cds.getValue()+"\t"+cds.getClient());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_duplicate(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ClientDefaultSetting cds = ClientDefaultSetting.getById(dc, id);
			Client c = cds.getClient();
			ClientDefaultSetting duplicate = c.newDefaultSetting(cds.getName(), cds.getValue());
			dc.saveItem(duplicate);
			dc.commit();
			dc.deleteItem(duplicate);
			dc.commit();
			throw new BaseException("Expected DatabaseException");
		}
		catch (DatabaseException ex)
		{
			write("--Duplicate client default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Duplicate client default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
