/*
	$Id: TestJep.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.util.jep.Jep;
import net.sf.basedb.util.jep.RawFunction;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Restriction;

public class TestJep
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing Java Math Expression Parser");

		String expression1 = "raw(ch1FgMean) - raw(ch1BgMean)";
		String expression2 = "ln((raw(ch1FgMean) - raw(ch1BgMean)) / (raw(ch2FgMean) - raw(ch2BgMean)))";
		String bigExpression = "1+sqrt(2)-exp(3)*abs(-4)/log(5)+ln(5.99)+x+if(a<b, 11, -11)";
		String restriction1 = expression1 + " > 0";
		String restriction2 = expression2 + " > 2";
		String restriction3 = restriction1 + " && " + restriction2;
		String restriction4 = "raw(name) == 'Nicklas'";
		String bigRestriction = "1 == 1 && 2 > 1 || 2 >= 1 && 3 < 7.0 && 3 <= 9 && !(5 != 5)";
		test_parse(expression1);
		test_parse(expression2);
		test_parse(bigExpression);
		test_parse(restriction1);
		test_parse(restriction2);
		test_parse(restriction3);
		test_parse(restriction4);
		test_parse(bigRestriction);
		test_expression(expression1);
		test_expression(expression2);
		test_expression(bigExpression);
		test_restriction(restriction1);
		test_restriction(restriction2);
		test_restriction(restriction3);
		test_restriction(restriction4);
		test_restriction(bigRestriction);

		write("++Testing Java Math Expression Parser "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_parse(String formula)
	{
		try
		{
			Jep.newJep(formula, new RawFunction(null, RawDataTypes.getSafeRawDataType("genepix"), true));
			write("--Parse formula: "+formula+" OK");
		}
		catch (Throwable ex)
		{
			write("--Parse formula: "+formula+" FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}
	
	static void test_expression(String formula)
	{
		try
		{
			Expression e = Jep.formulaToExpression(formula, new RawFunction(null, RawDataTypes.getSafeRawDataType("genepix"), true));
			write(e);
			write("--Jep to Expression: "+formula+" OK");
		}
		catch (Throwable ex)
		{
			write("--Jep to Expression: "+formula+" FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void test_restriction(String formula)
	{
		try
		{
			Restriction r = Jep.formulaToRestriction(formula, 
				new RawFunction(null, RawDataTypes.getSafeRawDataType("genepix"), true));
			write(r);
			write("--Jep to Restriction: "+formula+" OK");
		}
		catch (Throwable ex)
		{
			write("--Jep to Restriction: "+formula+" FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write(Expression e)
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("--Expression: " + e.toString());
		}
	}

	static void write(Restriction r)
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("--Restriction: " + r.toString());
		}
	}

}

