/*
	$Id: TestReporterFlatFileImporter.java 1376 2005-09-22 13:50:01 +0200 (Thu, 22 Sep 2005) nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import net.sf.basedb.core.plugin.*;

public class TestRawDataFlatFileImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing rawdata import using plugin");
		// Test import rawdata without lims connection
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		int fileId = TestFile.test_create("data/test.rawdata.import.txt", false, false);
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_import_from_file(arrayDesignId, "data/test.rawdata.import.txt", 
				"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", ".*(EMPTY_WELL|no clone).*", 0, 2, 1, 4);
		int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, arrayDesignId, 1, true);
		
		// Create plugin configuration and job
		int pluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.RawDataFlatFileImporter");
		int pluginConfigurationId = test_create_configuration(pluginDefinitionId);
		TestPluginConfiguration.test_load(pluginConfigurationId);
		int jobId = test_create_job(pluginConfigurationId, rawBioAssayId, fileId);
		
		// Execute job
		ok = ok && TestJob.test_execute(jobId);
		
		// Print result
		TestRawBioAssay.write_raw_data_header();
		TestRawBioAssay.test_list_raw_data(rawBioAssayId, 10, 5, 0, true);

		// Clean up test
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestRawBioAssay.test_delete(rawBioAssayId);
		TestArrayDesign.test_delete(arrayDesignId);
		TestJob.test_delete(jobId);
		TestPluginConfiguration.test_delete(pluginConfigurationId);
		TestFile.test_delete(fileId);
		TestReporter.test_delete();

		write("++Testing rawdata import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_configuration(int pluginDefinitionId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();
			write_request_information(request.getRequestInformation());
			request.setParameterValue("rawDataType", "genepix");
			PluginResponse response = request.invoke();

			if (response.getStatus() == Response.Status.ERROR)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}	
			else if (response.getStatus() == Response.Status.DONE)
			{
				throw new BaseException("Configuration should continue");
			}
			request = response.getNextRequest();
			write_request_information(request.getRequestInformation());
			
			request.setParameterValue("headerRegexp", "\"(.+)=(.*)\"");
			request.setParameterValue("dataSplitterRegexp", "\\t");
			request.setParameterValue("dataHeaderRegexp", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*");
			request.setParameterValue("blockColumnMapping", "\\Block\\");
			request.setParameterValue("columnColumnMapping", "\\Column\\");
			request.setParameterValue("rowColumnMapping", "\\Row\\");
			request.setParameterValue("reporterIdColumnMapping", "\\ID\\");
			request.setParameterValue("xColumnMapping", "\\5\\");
			request.setParameterValue("yColumnMapping", "\\6\\");
			
			request.setParameterValue("propertyMapping.diameter", "\\7\\");
			request.setParameterValue("propertyMapping.ch1FgMedian", "\\17\\");
			request.setParameterValue("propertyMapping.ch1FgMean", "\\18\\");
			request.setParameterValue("propertyMapping.ch1FgSd", "\\19\\");
			request.setParameterValue("propertyMapping.ch1BgMedian", "\\20\\");
			request.setParameterValue("propertyMapping.ch1BgMean", "\\21\\");
			request.setParameterValue("propertyMapping.ch1BgSd", "\\22\\");
			request.setParameterValue("propertyMapping.ch1PercSd1", "\\23\\");
			request.setParameterValue("propertyMapping.ch1PercSd2", "\\24\\");
			request.setParameterValue("propertyMapping.ch1PercSat", "\\25\\");
			
			request.setParameterValue("propertyMapping.ch2FgMedian", "\\8\\");
			request.setParameterValue("propertyMapping.ch2FgMean", "\\9\\");
			request.setParameterValue("propertyMapping.ch2FgSd", "\\10\\");
			request.setParameterValue("propertyMapping.ch2BgMedian", "\\11\\");
			request.setParameterValue("propertyMapping.ch2BgMean", "\\12\\");
			request.setParameterValue("propertyMapping.ch2BgSd", "\\13\\");
			request.setParameterValue("propertyMapping.ch2PercSd1", "\\14\\");
			request.setParameterValue("propertyMapping.ch2PercSd2", "\\15\\");
			request.setParameterValue("propertyMapping.ch2PercSat", "\\16\\");
			
			request.setParameterValue("propertyMapping.ratiosSd", "\\30\\");
			request.setParameterValue("propertyMapping.rgnRatio", "\\31\\");
			request.setParameterValue("propertyMapping.rgnR2", "\\32\\");
			request.setParameterValue("propertyMapping.fgPixels", "\\33\\");
			request.setParameterValue("propertyMapping.bgPixels", "\\34\\");
			request.setParameterValue("propertyMapping.flags", "\\46\\");
			//request.setParameterValue("minDataColumns", 48);
			//request.setParameterValue("maxDataColumns", 48);

			response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for rawdata import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for rawdata import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_job(int pluginConfigurationId, int rawBioAssayId, int fileId)
	{
		if (rawBioAssayId == 0 || pluginConfigurationId == 0 || fileId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(GuiContext.item(Item.RAWBIOASSAY));
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("rawBioAssay", RawBioAssay.getById(dc, rawBioAssayId));
			request.setParameterValue("numberFormatError", "null");
			request.setParameterValue("decimalSeparator", "dot");
			request.setParameterValue("duplicateFeatureError", "skip");
			request.setParameterValue("featureMismatchError", "smart");

			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				dc.saveItem(j);
				response.saveParameters(dc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for rawdata import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for rawdata import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}

	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
}

