/**
	$Id: TestWebservices.java 4740 2009-02-05 13:39:25Z nicklas $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.info.AnnotationInfo;
import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.ArrayDesignInfo;
import net.sf.basedb.info.BioAssaySetInfo;
import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.ExperimentInfo;
import net.sf.basedb.info.ExtendedPropertyInfo;
import net.sf.basedb.info.ProjectInfo;
import net.sf.basedb.info.QueryOptions;
import net.sf.basedb.info.QueryParameter;
import net.sf.basedb.info.RawBioAssayInfo;
import net.sf.basedb.info.ReporterListInfo;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.ws.client.AnnotationTypeClient;
import net.sf.basedb.ws.client.ArrayDesignClient;
import net.sf.basedb.ws.client.BioAssaySetClient;
import net.sf.basedb.ws.client.ExperimentClient;
import net.sf.basedb.ws.client.ProjectClient;
import net.sf.basedb.ws.client.RawBioAssayClient;
import net.sf.basedb.ws.client.ReporterClient;
import net.sf.basedb.ws.client.ServiceFactory;
import net.sf.basedb.ws.client.SessionClient;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;

/**
 	@author Martin, Nicklas
  	@since 2.5
 */
public class TestWebservices 
{
	static boolean ok = true;
	
	/**	
	  	@param args
	*/
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	
	static boolean test_all()
	{
		write("++Testing webservices");
		
		//Tests
		SessionClient client = test_SessionClient(null);
		if (client == null) return ok;
		
		//Create items to use in the tests
		
		int projectId = TestProject.test_create(true);
		int experimentId1 = TestExperiment.test_create("genepix", true);
		
		//Experiment with rawdata and array design stored in database.
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", 
				"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		int reporterListId1 = TestReporterList.test_create("net.sf.basedb.clients.test.reporterlist1", true);
		TestReporterList.test_add_reporters(reporterListId1, 50, 0);
		TestReporterList.test_setExperiment(reporterListId1, experimentId1);
				
		int arrayDesignId1 = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_import_from_file(arrayDesignId1, "data/test.rawdata.import.txt", 
				"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", null, 0, 2, 1, 4);		
		int rawBioAssayId1 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, arrayDesignId1, 1, false);
		int rawBioAssayId2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, arrayDesignId1, 1, false);
		TestRawBioAssay.test_import_from_file(rawBioAssayId1, "data/test.rawdata.import.txt");		
		TestRawBioAssay.test_import_from_file(rawBioAssayId2, "data/test.rawdata.import.txt");
		
		TestExperiment.test_add_rawbioassay(experimentId1, rawBioAssayId1);
		TestExperiment.test_add_rawbioassay(experimentId1, rawBioAssayId2);
		int rootBioAssaySetId1 = TestExperiment.test_create_root_bioassayset_using_calculator(experimentId1, "mean", rawBioAssayId1, rawBioAssayId2);
		
		//Experiment with rawdata and array design stored in files.
		int cdfId = TestFile.test_create("data/test.affymetrix.cdf", false, false);
		int celId = TestFile.test_create("data/test.affymetrix.cel", false, false);
		int arrayDesignId2 = TestArrayDesign.test_create(Platform.AFFYMETRIX, true);
		TestArrayDesign.test_set_file(arrayDesignId2, DataFileType.AFFYMETRIX_CDF, cdfId);
		int rawBioAssayId3 = TestRawBioAssay.test_create(Platform.AFFYMETRIX, null, null, 0, 0, 0, arrayDesignId2, 1, false);
		TestRawBioAssay.test_set_file(rawBioAssayId3, "affymetrix.cel", celId);
		int rawBioAssayId4 = TestRawBioAssay.test_create(Platform.AFFYMETRIX, null, null, 0, 0, 0, arrayDesignId2, 1, false);
		TestRawBioAssay.test_set_file(rawBioAssayId4, "affymetrix.cel", celId);
		
		int experimentId2 = TestExperiment.test_create("platform.affymetrix", true);
		TestExperiment.test_add_rawbioassay(experimentId2, rawBioAssayId3);
		TestExperiment.test_add_rawbioassay(experimentId2, rawBioAssayId4);
		
		int factorId1 = TestAnnotationType.test_create("Drug resistance", Type.STRING, 0, null, null, Item.RAWBIOASSAY, 1, new String[] { "high", "medium", "low" }, false);
		int factorId2 = TestAnnotationType.test_create("Temperature", Type.FLOAT, 0, null, null, Item.RAWBIOASSAY, 1, null, false);
		
		TestExperiment.test_add_experimental_factors(experimentId1, factorId1, factorId2);
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssayId1, factorId1, 0, "medium");
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssayId1, factorId2, 0, 12.5f);
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssayId2, factorId1, 0, "high");
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssayId2, factorId2, 0, 20.3f);
		
		//Tests
		test_ProjectClient(client, projectId);
		test_AnnotationTypes(client);
		test_getRawDataInExperiment(client, experimentId1, rawBioAssayId1);
		test_getRawDataInExperiment(client, experimentId2, rawBioAssayId3);	

		//Test to download a file which name in BASE starts with a number.
		TestFile.test_rename(celId, "2rawdataFile.txt");
		test_getRawDataInExperiment(client, experimentId2, rawBioAssayId3);
		
		test_getArrayDesign(client, rawBioAssayId1, arrayDesignId1);
		test_getArrayDesign(client, rawBioAssayId3, arrayDesignId2);		
		test_BioAssaySetClient(client, experimentId1, rootBioAssaySetId1);
		test_getExperimentalFactorValues(client, experimentId1);
		test_ReportersInExperiment(client, experimentId1, reporterListId1);
		test_Logout(client);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		//Standard test:Delete/cleanup
		TestReporterList.test_delete(reporterListId1);
		
		TestExperiment.test_delete_bioassayset(rootBioAssaySetId1);
		TestExperiment.test_delete(experimentId1);
		TestExperiment.test_delete(experimentId2);
		TestProject.test_delete(projectId);
		
		TestRawBioAssay.test_delete(rawBioAssayId1);
		TestRawBioAssay.test_delete(rawBioAssayId2);
		TestRawBioAssay.test_delete(rawBioAssayId3);
		TestRawBioAssay.test_delete(rawBioAssayId4);
		
		TestArrayDesign.test_delete(arrayDesignId1);
		TestArrayDesign.test_delete(arrayDesignId2);
		TestFile.test_delete(cdfId);
		TestFile.test_delete(celId);
		
		TestAnnotationType.test_delete(factorId1);
		TestAnnotationType.test_delete(factorId2);
		
		TestReporter.test_delete();		
		
		write("++Testing webservices "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void write(String message)
	{
		System.out.println(message);
	}

	/**
	 	Test for the BioAssaySetClient.
	 	<ul>
	 		<li>Get bioassaysets in an experiment
	 		<li>Get available export formats
	 		<li>Download BioAssaySet in each available format.
	 	</ul>
	  	@param client A logged in client.
	  	@param bioAssaySetId Id of the BioAssaySet to test with. 
	 */
	static void test_BioAssaySetClient(SessionClient client, int experimentId, int bioAssaySetId)
	{
		if (client == null || bioAssaySetId == 0) return;
		BioAssaySetClient basClient = new BioAssaySetClient(client);
		ExperimentClient experimentClient = new ExperimentClient(client);
		DbControl dc = null;		
		try
		{
			BioAssaySetInfo[] basInfo = experimentClient.getBioAssaySets(experimentId, new QueryOptions());
			if (basInfo == null) 
			{
				write ("--Get BioAssaySets from Experiment FAILED");
				ok = false;
			}
		}
		catch (Throwable t)
		{
			write ("--Get BioAssaySets from Experiment FAILED");
			t.printStackTrace();
			ok = false;
		}
		
		try
		{
			String[] exportFormats = basClient.getExportFormats(); 
			if (exportFormats == null) 
			{
				write ("--BioAssaySetClient: Getting availble export formats FAILED");
				ok = false;
			}
			else
			{
			
				dc = TestUtil.getDbControl();
				BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
				
				ArrayList<String> rep = new ArrayList<String>();
				rep.add("rep.id");
				rep.add("rep.name");
				rep.add("rep.position");
				
				ArrayList<String> spot = new ArrayList<String>();
				ItemQuery<Formula> ratio = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, bas.getRawDataType());
				ratio.restrict(Restrictions.eq(Hql.property("name"), Expressions.string("Ratio, ch1 / ch2")));
				if (ratio.count(dc) < 1)
				{
					throw new Exception("Cant find formula 'Ratio, ch1 / ch2'");
				}
				spot.add("intensity1");
				spot.add("raw.row");
				spot.add("raw.col");
				spot.add("raw.flags");
				spot.add("frm."+ratio.list(dc).get(0).getId());

				String[] reporterFields = new String[rep.size()];
				String[] spotFields = new String[spot.size()];
				boolean mergeReporters = false;
				
				for (String format : exportFormats)
				{
					File receivedFile = new File(format);
					OutputStream out = new FileOutputStream(receivedFile);
					System.out.print("--Download BioAssaySet in file format " + format + "...");
					InputStream in = basClient.downloadBioAssaySet(
							bioAssaySetId, format, rep.toArray(reporterFields), spot.toArray(spotFields), mergeReporters);
					if (in == null)
					{
						write ("FAILED InputStream==null");
						ok = false;
					}
					else
					{
						FileUtil.copy(in, out);
						in.close();
						out.close();
						write("OK; size=" + receivedFile.length() + " bytes");
						receivedFile.delete();
					}
				}
			}
			write("--Test of BioAssaySetClient OK");
		}
		catch (Throwable t)
		{
			write ("--Test of BioAssaySetClient FAILED");
			t.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null)dc.close();
		}
	}
	
	/**
	 	Following items are tested in raw bioassay service and array design service.
	 	<ul>
	 		<li>Get info about an array design used in a rawBioAssayId
	 		<li>Check if an array design has downloadable data.
	 		<li>Get the data file types used in an array design
	 		<li>Download the first of the data file-types.
	 	</ul>
	  	@param client A logged in client.
	  	@param rawBioAssayId Id of the RawBioAssay to test with.
	  	@param arrayDesignId Id of the ArrayDesign to test with
	 */
	static void test_getArrayDesign(SessionClient client, int rawBioAssayId, int arrayDesignId)
	{
		if (client == null || rawBioAssayId == 0 || arrayDesignId == 0) return;
		ArrayDesignClient arrayDesignClient = new ArrayDesignClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);		
		ArrayDesignInfo designInfo = null;
		
		try
		{					
			designInfo = rawClient.getArrayDesign(rawBioAssayId);			
		}
		catch (Throwable t)
		{
			write ("--Get info about RawBioAssay's ArrayDesign FAILED");
			t.printStackTrace();
			ok = false;
		}
		if (designInfo != null)
		{
			try
			{
				boolean hasDownloadableData = arrayDesignClient.hasDownloadableData(arrayDesignId);
				DataFileTypeInfo[] fileTypes = arrayDesignClient.getDataFileTypes(arrayDesignId, new QueryOptions());
				
				if (hasDownloadableData && fileTypes.length > 0)
				{
					System.out.print("--Download of " + fileTypes[0].getName() + " for array design....");
					File receivedFile = new File(fileTypes[0].getName());
					OutputStream out = new FileOutputStream(receivedFile);
					InputStream in = arrayDesignClient.downloadArrayDesignByType(arrayDesignId, fileTypes[0].getExternalId());
					if (in == null)
					{
						write ("FAILED InputStream==null");
						ok = false;
					}
					else
					{
						FileUtil.copy(in, out);
						in.close();
						out.close();
						write("OK; size=" + receivedFile.length() + " bytes");
						receivedFile.delete();
					}
				}
			}
			catch (Throwable t)
			{
				write ("--Test of ArrayDesignClient FAILED");
				t.printStackTrace();
				ok = false;
			}
		}
		else write ("--No info about the Raw BioAssay was received");
		write ("--Test of ArrayDesignClient OK");
	}
	
	/**
	 	This tests following in experiment service and raw bioassay service:
	 	<ul>
	 		<li>Get experiment with a specific id
	 		<li>Get rawbioassay with a specific id from an experiment.
	 		<li>Checks if the rawbioassay has downloadable data.
	 		<li>Get the filetypes used in the raw bioassay.
	 		<li>Download raw data for the first filetype.
	 	</ul>
	  	@param client A logged in client.
	  	@param experimentId Id of the Experiment to test with. 
	  	@param rawBioAssayId Id of the RawBioAssay to test with.
	 */
	static void test_getRawDataInExperiment(SessionClient client, int experimentId, int rawBioAssayId)
	{
		if (client == null || experimentId == 0 || rawBioAssayId == 0)return;
		
		ExperimentClient experimentClient = new ExperimentClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);
		QueryOptions qOpt = new QueryOptions();
		try
		{
			qOpt.setRestrictionWithPrefix("$id="+experimentId, "$");
			ExperimentInfo[] experimentInfo = experimentClient.getExperiments(qOpt);
			qOpt = null;
			if (experimentInfo.length < 1) throw new Exception("Experiment was not received");			
		}
		catch (Throwable t)
		{
			write ("--Get experiment with id " + experimentId + " FAILED");
			t.printStackTrace();
			ok = false;
		}
		
		try
		{
			qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$id='" + rawBioAssayId+"'", "$");
			
			RawBioAssayInfo[] rawInfo = experimentClient.getRawBioAssays(experimentId, qOpt);
			if (rawInfo.length < 1) 
			{
				write ("--No RawBioAssays were received");
				return;
			}
			
			Boolean hasDownloadableData = rawClient.hasDownloadableData(rawBioAssayId);
			DataFileTypeInfo[] fileTypes = rawClient.getDataFileTypes(rawBioAssayId, null);
			
			if (hasDownloadableData && fileTypes.length > 0)
			{
				System.out.print("--Download of " + fileTypes[0].getName() + " for RawBioAssay....");
				File receivedFile = new File(fileTypes[0].getName());
				OutputStream out = new FileOutputStream(receivedFile);
				InputStream in = rawClient.downloadRawDataByType(rawBioAssayId, fileTypes[0].getExternalId());
				if (in == null)
				{
					write ("FAILED InputStream==null");
					ok = false;
				}
				else
				{
					FileUtil.copy(in, out);
					in.close();
					out.close();
					write("OK; size=" + receivedFile.length() + " bytes");
					receivedFile.delete();
				}
			}
			write ("--Test of RawBioAssayClient OK");
		}
		catch(Throwable t)
		{
			write ("--Test of RawBioAssayClient FAILED");
			t.printStackTrace();
			ok = false;
		}
		
	}
	
	
	/**
	 	This tests getting the experimental factors for an experiment and
	 	listing values for each raw bioassay.
	  	@param client A logged in client.
	  	@param experimentId Id of the Experiment to test with. 
	*/
	static void test_getExperimentalFactorValues(SessionClient client, int experimentId)
	{
		if (client == null || experimentId == 0) return;
		
		ExperimentClient experimentClient = new ExperimentClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);
		QueryOptions qOpt = new QueryOptions();

		try
		{
			AnnotationTypeInfo[] factors = experimentClient.getExperimentalFactors(experimentId, qOpt);
			RawBioAssayInfo[] rawInfo = experimentClient.getRawBioAssays(experimentId, qOpt);

			StringBuilder header = new StringBuilder("\t");
			Integer[] annotationTypes = new Integer[factors.length];
			for (int i = 0; i < factors.length; ++i)
			{
				annotationTypes[i] = factors[i].getId();
				header.append(factors[i].getName()).append("\t");
			}
			
			write(header.toString());
			for (RawBioAssayInfo rba : rawInfo)
			{
				StringBuilder row = new StringBuilder();
				row.append(rba.getName()).append("\t");
				AnnotationInfo[] annotations = rawClient.getAnnotationValues(rba.getId(), 
					annotationTypes, true);
				
				for (AnnotationInfo info : annotations)
				{
					row.append(Arrays.asList(info.getValues()).toString()).append("\t");
				}
				write(row.toString());
			}
			
			write ("--Test of getExperimentalFactorValues OK");
		}
		catch(Throwable t)
		{
			write ("--Test of getExperimentalFactorValues FAILED");
			t.printStackTrace();
			ok = false;
		}
		
	}	
	
	static void test_ProjectClient(SessionClient client, int projectId)
	{
		if (client == null || projectId == 0) return;
		
		ProjectClient projectClient = new ProjectClient(client);
		try
		{
			//Tests ProjectClient.getProjects(QueryOptions qOpt)
			QueryOptions qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$id="+projectId, "$");
			ProjectInfo[] infos = projectClient.getProjects(qOpt);
			if (!(infos != null && infos.length > 0))
			{				
				throw new Exception("No project was received");
			}
			
			//Tests ProjectClient.setActive(int projectId)
			projectClient.setActive(projectId);			
			write("--Test of ProjectClient OK");
		}		
		catch (Throwable t)
		{
			write ("--Test of ProjectClient FAILED");
			t.printStackTrace();
			ok = false;
		}		
	}
	
	static void test_AnnotationTypes(SessionClient client)
	{
		if (client == null) return;
		
		AnnotationTypeClient atClient = new AnnotationTypeClient(client);
		try
		{
			//Tests ProjectClient.getProjects(QueryOptions qOpt)
			QueryOptions qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$valueType = :vt", "$");
			qOpt.setParameters(new QueryParameter[]{ new QueryParameter("vt", Type.STRING.getValue()) });
			AnnotationTypeInfo[] infos = atClient.getAnnotationTypes(null, qOpt);
			
			for (AnnotationTypeInfo info : infos)
			{
				write(info.getId() +"\t" + info.getName() + "\t" + info.getValueType() + "\t" + 
					Arrays.asList(info.getItemTypes()));
			}
			
			write("--Test of AnnotationTypeClient OK");
		}
		catch (Throwable t)
		{
			write ("--Test of AnnotationTypeClient FAILED");
			t.printStackTrace();
			ok = false;
		}		
	}	
	
	static SessionClient test_SessionClient(ServiceFactory serviceFactory)
	{
		SessionClient client = null;
		String url = Config.getString("test.webservices.url");
		if (url == null)
		{
			write("--Skipping webservices test: setting base.config[test.webservices.url] not found");
			return null;
		}
		try
		{
			write("--Using url: " + url);
			client = new SessionClient(url, null, TestUtil.getClient());
			client.login(TestUtil.getLogin(), TestUtil.getPassword(), "Test webservices", false);
	        write("--Create/login session OK: ID=" + client.getId());
		}
		catch (Throwable t)
		{
			write("--Create/login session FAILED");
			write("--Skipping webservices test.");
			t.printStackTrace();
			return null;
		}
		try
		{
			client.refreshSession(client.getId());
			write("--Refresh session OK \tSession timeout set to: " + client.getSessionTimeOut() + " minutes");
		}
		catch(Throwable t)
		{
			write ("--Refresh session or get session's timeout FAILED");
			t.printStackTrace();
			ok = false;
		}		
		return client;
	}
	
	static void test_Logout(SessionClient client)
	{
		if (client == null) return;
		try
		{
			client.logout();
	        write("--Logout OK");
		}
		catch (Throwable t)
		{
			write("--Logout FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	static void test_ReportersInExperiment(SessionClient client, int experimentId, int reporterListId)
	{
		if (client == null || experimentId == 0 || reporterListId == 0)return;
		ExperimentClient experimentClient = new ExperimentClient(client);
		ReporterClient reporterClient = new ReporterClient(client);
		try
		{
			ReporterListInfo[] listInfo = experimentClient.getReporterLists(experimentId, null);
			write(listInfo.length > 0 ? 
					"--Received reporterlists(" + listInfo.length + ")" : 
					"--No reporterlists were received");
			
			ExtendedPropertyInfo[] extPropertyInfos = reporterClient.getExtendedProperties();
			write(extPropertyInfos.length > 0 ?
					"--Received extended properties for reporters(" + extPropertyInfos.length + ")" : 
					"--No extended properties were received");
			
			if (listInfo.length > 0 && extPropertyInfos.length > 0)
			{				
				System.out.print("--Download of reporters in reporterlist '" + listInfo[0].getName() +"'...." );
				File receivedFile = new File(listInfo[0].getName() + ".txt");
				OutputStream out = new FileOutputStream(receivedFile);
				 
				InputStream in = reporterClient.downloadReportersInList(listInfo[0].getId(), extPropertyInfos, new QueryOptions());
				
				if (in == null)
				{
					write ("FAILED InputStream==null");
					ok = false;
				}
				else
				{
					FileUtil.copy(in, out);
					in.close();
					out.close();
					write("OK; size=" + receivedFile.length() + " bytes");
					receivedFile.delete();
				}
				
				receivedFile = null;
				out = null;
				
				//Test to receive reporters from service, at most 50 reporters				
				System.out.print("--Download of reporters....");				
				receivedFile = new File("ReporterFile.txt");
				out = new FileOutputStream(receivedFile);
				in = null;
				QueryOptions qOpt = new QueryOptions();
				in = reporterClient.downloadReporters(extPropertyInfos, qOpt);
				
				if (in == null)
				{
					write ("FAILED InputStream==null");
					ok = false;
				}
				else
				{
					FileUtil.copy(in, out);
					in.close();
					out.close();
					write("OK; size=" + receivedFile.length() + " bytes");
					receivedFile.delete();
				}
			}
				
			write("--Test of ReporterService OK");
		}
		catch (Throwable t)
		{
			write ("--Test of getReporterLists FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
}
