/**
	$Id: BioAssaySetClient.java 4513 2008-09-11 20:06:36Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import java.io.IOException;
import java.io.InputStream;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Web service client for BioAssaySetService in BASE.
 	{@link net.sf.basedb.ws.server.BioAssaySetService}
  	@author Martin
  	@version 2.5
 */
public class BioAssaySetClient 
	extends AbstractRPCClient
{
	private final SessionClient session;
	
	/**
	 	Creates new BioAssayClient for a session.
	  	@param session Active session.
	 */
	public BioAssaySetClient(SessionClient session)
	{
		super(session.getUrl(), "BioAssaySet", session.getServiceFactory());
		this.session = session;
	}
	
	/**
	 	Calls the BioAssaySetService and gets the available formats to export
	 	a BioAssaySet from BASE in.
	  	@return An array with the formats. 
	  	@throws AxisFault If communication with the service fails.
	 */
	public String[] getExportFormats()
		throws AxisFault
	{
		String[] formats = invokeBlocking("getExportFormats", String[].class, session.getId());
		return formats;
	}
	
	/**
	 	Call the service in BioAssaySetService that exports a bioassay set and
	 	then sends it as a file. 
	  	@param bioAssaySetId Id of the bioAssay set to export
	  	@param format Export format to use when exporting the bioassay set
	  	@param reporterFields Reporter fields to include in the export
	  	@param spotFields Spot fields to include in the export
	  	@param mergeReporters If the reporters should be merged or not.
	  	@return An InputStream to read the file data from
	  	@throws AxisFault If communication with services fails.
	  	@throws IOException If accessing the file files.
	 */
	public InputStream downloadBioAssaySet(int bioAssaySetId, String format, 
			String[] reporterFields, String[] spotFields, boolean mergeReporters) 
		throws AxisFault, IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadBioAssaySet",
				session.getId(), 
				bioAssaySetId, 
				format,
				reporterFields,
				spotFields,
				mergeReporters
			);			
	}
}
